/*
Variable PSO, research version
Coming from "Balanced PSO", by just keeping the two most promising options, i.e.:
- variable ring topology
- variable swarm size

Otherwise, the velocity update equation is the one of Standard PSO 2007.
The swarm size is bigger here, it is just an _initial_ swarm size, and it
can decrease.

 Contact for remarks, suggestions etc.:
 Maurice.Clerc@WriteMe.com
 PS. Don't tell me the code is not optimised, I know  ;-)
But if you find a bug, don't hesitate!


 Last update. The date is also the name of the version
2011-01-20 Fixed a bug (QU Xiaohui), added options for Rule1 and Rule2
2010-04-15


 ----------------------------------- Basic parameters
 S := swarm size
 w := first cognitive/confidence coefficient
 c := second cognitive/confidence coefficient

Suggested values are either hard coded or automatically computed.

 ----------------------------------- Equations of the "normal" move
 For each particle and each dimension
 Equation 1:	v(t+1) = w*v(t) + R(c1)*(p(t)-x(t)) + R(c2)*(g(t)-x(t))
 Equation 2:	x(t+1) = x(t) + v(t+1)
 where
 v(t) := velocity at time t
 x(t) := position at time t
 p(t) := best previous position of the particle
 g(t) := best position amongst the best previous positions
 of the informants of the particle
 R(c) := a number coming from a random distribution, which depends on c
 In SPSO 2007, the distribution is uniform on [0,c]

 Note 1:
 When the particle has no informant better than itself,
 it implies p(t) = g(t)

 Equation 1 is then modified:
 v(t+1) = w*v(t) + R(c)*(p(t)-x(t))  
and NOT   v(t+1) = w*v(t) + 2*R(c)*(p(t)-x(t))

 ----------------------------------- Miscellaneous
- the search space can be any real hyperparallelepid, but the values can be discretised
	(granularity)
- the search space can also be a list of values (reads from a file)

 */
 
 /* 
 A few results (formula=1; spreadProba=1, rule1=0, rule2=1)
 Function			Success_rate(over 100 runs)
 4						93%
 102					75%				
 8						72%
 9						20%
 */

#include "main.h" // Structures, subprogramme list, global variables
// =================================================
int main () 
{ 
	struct position bestBest;
	double convRateMean;
	int d;					// Current dimension
	double error;				// Current error
	double errorMean;			// Average error
	double errorMin=infinity;	// Best result over all runs
	double errorMeanBest[RMax]; 
	double evalMean;			// Mean number of evaluations
	int functionCode;

	int n;
	int nFailure;				// Number of unsuccessful runs
	double logProgressMean=0;
	struct param param={0};
	struct problem pb={0};
	int  runMax; 
	struct result result;
	double successRate;
	double variance;
	


	f_run = fopen ("f_run.txt", "w");  
	f_synth = fopen ("f_synth.txt", "w"); 
	f_expl = fopen ("f_expl.txt", "w"); // Just for info (exploitation rate)
	fLandscape=fopen("fLandscape.txt","r"); // When data are on a file
	f_init_save=fopen("f_init_save.txt","w");
	f_coeff= fopen("f_coeff.txt","w");
	f_swarm=fopen("f_swarm.txt","w");

	E = exp ((long double) 1); 
	pi = acos ((long double) -1);
	
	
	// ----------------------------------------------- PROBLEM
	functionCode = 4;
	/* (see problemDef, cellular_phone, cec2005pb for precise definitions)
	 0 Parabola (Sphere)
	 1 Griewank
	 2 Rosenbrock (Banana)
	 3 Rastrigin
	 4 Tripod (dimension 2)
	 5 Ackley
	 6 Center-bias test function
	 7 Pressure vessel (penalty method)
	 1007        "        (confinement method)
	 8 Compression spring
	 1008				"				 (confinement method)
	 9 Gear train
	 10 Cellular phone

	 12 Schwefel
	 13 Cutting stock
	 14 convex small polygon, 4 edges, smallest perimeter
	 15 constrained problem g13 (penalty method)
	 16 constrained problem g3 (penalty method)
	 17 Lennard-Jones problem
	 1015 				"               (confinement method)

	 CEC 2005 benchmark  (no more than 30D. See cec2005data.c)
	 100 F1 (shifted Parabola/Sphere) 
	 102 F6 (shifted Rosenbrock) 
	 103 F9 (shifted Rastrigin) 
	 104 F2 Schwefel 
	 105 F7 Griewank  (NOT rotated)
	 106 F8 Ackley  (NOT rotated) 

	 99 Test

	 */ 

	runMax = 100; // Numbers of runs


	// =========================================================== 

		pb=problemDef(functionCode,fLandscape);

		  #include "paramVPSO.c"
														
	// Display information about the problem and the parameters
		// Some information
		printf ("\n Function %i ", functionCode);
		printf("\n Dimension %i",pb.SS.D);


		//---------------
		convRateMean=0;
		errorMean = 0;	    
		evalMean = 0;	    
		nFailure = 0;	
		
		//------------------------------------- RUNS	
		for (run = 0; run < runMax; run++)  
	{	
		//printf("\n run %i/%i",run+1,runMax); 
				// For some options, positions must be memorised as much a possible
		fprintf(f_swarm,"run %i\n",run);

		result = PSO (param, pb, 0);
		error = errorFC(result.error);

		convRateMean=convRateMean+result.convRate;

		if (error > pb.epsilon) // Failure
		{
			nFailure = nFailure + 1;
		}

		// Result display
		printf("\nmean %1.2e", errorMean/(run+1)); // Temporary error mean
		printf(", %0.0f %% ", (double)100*(run+1-nFailure)/(run+1)); // Temporary success rate
		printf (" %i/%i. Eval %f. Error %f ", run+1,runMax, result.nEval, error);
		printf("  x(0)= %f",(double)result.SW.P[result.SW.best].x[0]);

		// Save result
		fprintf( f_run, "\n%i %f %e ", run, result.nEval,  error );
		for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_run, " %0.20f",  (double)result.SW.P[result.SW.best].x[d] );


		// Compute/store some statistical information
		if (run == 0)
		{
			errorMin = error;
			bestBest=result.SW.P[result.SW.best];
		}
		else if (error < errorMin)
		{
			errorMin = error;
			bestBest=result.SW.P[result.SW.best];
		}

		evalMean = evalMean + result.nEval;	
		errorMean = errorMean + error;	
		errorMeanBest[run] = error;
		logProgressMean  = logProgressMean - log(error);

	}		// End loop on "run"


		// Display statistical information
		// Means
		convRateMean=convRateMean/ (double) runMax; 
		evalMean = evalMean / (double) runMax;   
		errorMean = errorMean / (double) runMax;
		logProgressMean = logProgressMean/(double) runMax;

		printf ("\n Conv. rate (mean)= %f", convRateMean);		
		printf ("\n Eval. (mean)= %f", evalMean);	
		printf ("\n Error (mean) = %.12f, %.20e", errorMean,errorMean);

		// Variance
		variance = 0;

		for (run = 0; run < runMax; run++)
			variance = variance + pow (errorMeanBest[run] - errorMean, 2);

		variance = sqrt (variance / runMax);	    
		printf ("\n Std. dev. %.20f", variance); 
		printf("\n Log_progress (mean) = %f", logProgressMean);	

		// Success rate and minimum value
		printf("\n Failure(s) %i",nFailure);
		successRate = 1 - nFailure / (double) runMax;			
		printf ("\n Success rate = %.2f%%", 100*successRate);

		//if (run > 1)
		printf ("\n Best min value = %f", bestBest.f.f[0]);
		if(pb.constraint>0)
	{
		printf("%i constraints (should be negative): ",pb.constraint);
		for(n=0;n<pb.constraint;n++) printf("%f ",bestBest.f.f[n+1]);
	}

		// Display the best position
		printf("\n Position: ");
		for ( d = 0; d < pb.SS.D; d++ ) printf( " %f", (double) bestBest.x[d] );

		// Save
		/*
		 fprintf(f_synth,"\n"); for (d=0;d<SS.D;d++) fprintf(f_synth,"%f ",
		     pb.offset[d]);
		 fprintf(f_synth,"    %f %f %f %.0f%% %f",errorMean,variance,errorMin,
		     successRate,evalMean); 

		 fprintf( f_synth, "\n%f %f %f %f %.0f%% %f ", shift,
		     errorMean, variance, errorMin, successRate, evalMean );
		 */	 	
		fprintf (f_synth, "%f %f %.0f%% %f   ",
		    errorMean, variance, 100*successRate, evalMean);		   

		// Save the best result
		fprintf(f_synth,"\n ");
		for(n=0;n<pb.constraint+1;n++) fprintf( f_synth, "%f  ", bestBest.f.f[n]);
		fprintf(f_synth," X");
		for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_synth, " %f", (double) bestBest.x[d] );
	

		//-----------------------------------------------------
		// Repeat some information
		printf("\n\n Function %i", functionCode); printf(" Dimension %i",pb.SS.D);
		
		return 0; // End of main program
}



// =============================================================================
#include "alea.c"
#include "compare.c"
#include "confin.c"
#include "init.c"
#include "move.c"
#include "perf.c"
#include "problemDef.c"
#include "tools.c"
#include "PSO.c"





